@@ -5,8 +5,6 @@ __all__ = ['request', 'HttpClient']
 import asyncio
 import base64
 import collections
-import email.message
-import http.client
 import http.cookies
 import json
 import io
@@ -23,6 +21,10 @@ import weakref
 import warnings
 
 import aiohttp
+from .multidict import CaseInsensitiveMultiDict, MultiDict, MutableMultiDict
+
+HTTP_PORT = 80
+HTTPS_PORT = 443
 
 
 @asyncio.coroutine
@@ -37,14 +39,14 @@ def request(method, url, *,
             max_redirects=10,
             encoding='utf-8',
             version=(1, 1),
-            timeout=None,
             compress=None,
             chunked=None,
             expect100=False,
             connector=None,
             loop=None,
             read_until_eof=True,
-            request_class=None):
+            request_class=None,
+            response_class=None):
     """Constructs and sends a request. Returns response object.
 
     :param method: http method
@@ -59,7 +61,6 @@ def request(method, url, *,
     :param files: (optional) Dictionary of 'name': file-like-objects
        for multipart encoding upload
     :param auth: (optional) Auth tuple to enable Basic HTTP Auth
-    :param timeout: (optional) Float describing the timeout of the request
     :param allow_redirects: (optional) Boolean. Set to True if POST/PUT/DELETE
        redirect following is allowed.
     :param compress: Boolean. Set to True if request has to be compressed
@@ -72,6 +73,7 @@ def request(method, url, *,
     :param read_until_eof: Read response until eof if response
        does not have Content-Length header.
     :param request_class: Custom Request class implementation.
+    :param response_class: Custom Response class implementation.
     :param loop: Optional event loop.
 
     Usage::
@@ -79,7 +81,7 @@ def request(method, url, *,
       >>> import aiohttp
       >>> resp = yield from aiohttp.request('GET', 'http://python.org/')
       >>> resp
-      <HttpResponse(python.org/) [200]>
+      <ClientResponse(python.org/) [200]>
       >>> data = yield from resp.read_and_close()
 
     """
@@ -87,34 +89,28 @@ def request(method, url, *,
     if loop is None:
         loop = asyncio.get_event_loop()
     if request_class is None:
-        request_class = HttpRequest
+        request_class = ClientRequest
     if connector is None:
         connector = aiohttp.TCPConnector(loop=loop)
 
     while True:
-        using_proxy = isinstance(connector, aiohttp.ProxyConnector)
         req = request_class(
             method, url, params=params, headers=headers, data=data,
             cookies=cookies, files=files, auth=auth, encoding=encoding,
             version=version, compress=compress, chunked=chunked,
-            loop=loop, expect100=expect100, using_proxy=using_proxy)
+            loop=loop, expect100=expect100,
+            response_class=response_class)
 
         try:
             conn = yield from connector.connect(req)
 
             resp = req.send(conn.writer, conn.reader)
             try:
-                if timeout:
-                    yield from asyncio.wait_for(
-                        resp.start(conn, read_until_eof), timeout, loop=loop)
-                else:
-                    yield from resp.start(conn, read_until_eof)
+                yield from resp.start(conn, read_until_eof)
             except:
                 resp.close()
                 conn.close()
                 raise
-        except asyncio.TimeoutError:
-            raise aiohttp.TimeoutError from None
         except aiohttp.BadStatusLine as exc:
             raise aiohttp.ClientConnectionError(exc)
         except OSError as exc:
@@ -127,7 +123,7 @@ def request(method, url, *,
                 resp.close()
                 break
 
-            r_url = resp.get('location') or resp.get('uri')
+            r_url = resp.headers.get('LOCATION') or resp.headers.get('URI')
 
             scheme = urllib.parse.urlsplit(r_url)[0]
             if scheme not in ('http', 'https', ''):
@@ -145,20 +141,21 @@ def request(method, url, *,
     return resp
 
 
-class HttpRequest:
+class ClientRequest:
 
     GET_METHODS = {'DELETE', 'GET', 'HEAD', 'OPTIONS'}
     POST_METHODS = {'PATCH', 'POST', 'PUT', 'TRACE'}
     ALL_METHODS = GET_METHODS.union(POST_METHODS)
 
     DEFAULT_HEADERS = {
-        'Accept': '*/*',
-        'Accept-Encoding': 'gzip, deflate',
+        'ACCEPT': '*/*',
+        'ACCEPT-ENCODING': 'gzip, deflate',
     }
 
     body = b''
     auth = None
     response = None
+    response_class = None
 
     _writer = None  # async task for streaming data
     _continue = None  # waiter future for '100 Continue' response
@@ -169,14 +166,14 @@ class HttpRequest:
     # doesn't produce request finalization.
     # After .write_bytes is done _writer has set to None and we have nothing
     # to cancel.
-    # Maybe we need to add .cancel() method to HttpRequest through for
+    # Maybe we need to add .cancel() method to ClientRequest through for
     # forced closing request sending.
 
     def __init__(self, method, url, *,
                  params=None, headers=None, data=None, cookies=None,
                  files=None, auth=None, encoding='utf-8', version=(1, 1),
                  compress=None, chunked=None, expect100=False,
-                 verify_ssl=True, loop=None, using_proxy=False):
+                 verify_ssl=True, loop=None, response_class=None):
         self.url = url
         self.method = method.upper()
         self.encoding = encoding
@@ -184,7 +181,7 @@ class HttpRequest:
         self.compress = compress
         self.verify_ssl = verify_ssl
         self.loop = loop
-        self.using_proxy = using_proxy
+        self.response_class = response_class or ClientResponse
 
         self.update_version(version)
         self.update_host(url)
@@ -222,6 +219,9 @@ class HttpRequest:
             if len(self.auth) == 1:
                 self.auth.append('')
 
+        # Record entire netloc for usage in host header
+        self.netloc = netloc
+
         # extract host and port
         self.ssl = scheme == 'https'
         if ':' in netloc:
@@ -233,9 +233,9 @@ class HttpRequest:
                     'Port number could not be converted.') from None
         else:
             if self.ssl:
-                self.port = http.client.HTTPS_PORT
+                self.port = HTTPS_PORT
             else:
-                self.port = http.client.HTTP_PORT
+                self.port = HTTP_PORT
 
         self.scheme = scheme
         self.host = netloc
@@ -278,31 +278,28 @@ class HttpRequest:
             else:
                 query = params
 
-        if not self.using_proxy:
-            scheme = ''
-            netloc = ''
-
         self.path = urllib.parse.urlunsplit(
-            (scheme, netloc,
-             urllib.parse.quote(path, safe='/%'), query, fragment))
+            ('', '', urllib.parse.quote(path, safe='/%'), query, fragment))
 
     def update_headers(self, headers):
         """Update request headers."""
-        self.headers = email.message.Message()
+        self.headers = MutableMultiDict()
         if headers:
             if isinstance(headers, dict):
                 headers = headers.items()
+            elif isinstance(headers, MultiDict):
+                headers = headers.items(getall=True)
 
             for key, value in headers:
-                self.headers.add_header(key, value)
+                self.headers.add(key.upper(), value)
 
         for hdr, val in self.DEFAULT_HEADERS.items():
             if hdr not in self.headers:
                 self.headers[hdr] = val
 
         # add host
-        if 'host' not in self.headers:
-            self.headers['Host'] = self.host
+        if 'HOST' not in self.headers:
+            self.headers['HOST'] = self.netloc
 
     def update_cookies(self, cookies):
         """Update request cookies header."""
@@ -310,9 +307,9 @@ class HttpRequest:
             return
 
         c = http.cookies.SimpleCookie()
-        if 'cookie' in self.headers:
-            c.load(self.headers.get('cookie', ''))
-            del self.headers['cookie']
+        if 'COOKIE' in self.headers:
+            c.load(self.headers.get('COOKIE', ''))
+            del self.headers['COOKIE']
 
         if isinstance(cookies, dict):
             cookies = cookies.items()
@@ -324,18 +321,18 @@ class HttpRequest:
             else:
                 c[name] = value
 
-        self.headers['cookie'] = c.output(header='', sep=';').strip()
+        self.headers['COOKIE'] = c.output(header='', sep=';').strip()
 
     def update_content_encoding(self):
         """Set request content encoding."""
-        enc = self.headers.get('content-encoding', '').lower()
+        enc = self.headers.get('CONTENT-ENCODING', '').lower()
         if enc:
             self.compress = enc
             self.chunked = True  # enable chunked, no need to deal with length
         elif self.compress:
             if not isinstance(self.compress, str):
                 self.compress = 'deflate'
-            self.headers['content-encoding'] = self.compress
+            self.headers['CONTENT-ENCODING'] = self.compress
             self.chunked = True  # enable chunked, no need to deal with length
 
     def update_auth(self, auth):
@@ -346,7 +343,7 @@ class HttpRequest:
         if auth:
             if isinstance(auth, (tuple, list)) and len(auth) == 2:
                 # basic auth
-                self.headers['Authorization'] = 'Basic %s' % (
+                self.headers['AUTHORIZATION'] = 'Basic %s' % (
                     base64.b64encode(
                         ('%s:%s' % (auth[0], auth[1])).encode('latin1'))
                     .strip().decode('latin1'))
@@ -357,13 +354,13 @@ class HttpRequest:
         if (hasattr(data, '__iter__') and not isinstance(
                 data, (bytes, bytearray, str, list, dict))):
             self.body = data
-            if 'content-length' not in self.headers and self.chunked is None:
+            if 'CONTENT-LENGTH' not in self.headers and self.chunked is None:
                 self.chunked = True
         else:
             if isinstance(data, (bytes, bytearray)):
                 self.body = data
-                if 'content-type' not in self.headers:
-                    self.headers['content-type'] = 'application/octet-stream'
+                if 'CONTENT-TYPE' not in self.headers:
+                    self.headers['CONTENT-TYPE'] = 'application/octet-stream'
             else:
                 # form data (x-www-form-urlencoded)
                 if isinstance(data, dict):
@@ -374,12 +371,12 @@ class HttpRequest:
 
                 self.body = data.encode(self.encoding)
 
-                if 'content-type' not in self.headers:
-                    self.headers['content-type'] = (
+                if 'CONTENT-TYPE' not in self.headers:
+                    self.headers['CONTENT-TYPE'] = (
                         'application/x-www-form-urlencoded')
 
-            if 'content-length' not in self.headers and not self.chunked:
-                self.headers['content-length'] = str(len(self.body))
+            if 'CONTENT-LENGTH' not in self.headers and not self.chunked:
+                self.headers['CONTENT-LENGTH'] = str(len(self.body))
 
     def update_body_from_files(self, files, data):
         """Generate multipart/form-data body."""
@@ -424,18 +421,18 @@ class HttpRequest:
         self.body = encode_multipart_data(
             fields, bytes(boundary, 'latin1'))
 
-        self.headers['content-type'] = (
+        self.headers['CONTENT-TYPE'] = (
             'multipart/form-data; boundary=%s' % boundary)
 
     def update_transfer_encoding(self):
         """Analyze transfer-encoding header."""
-        te = self.headers.get('transfer-encoding', '').lower()
+        te = self.headers.get('TRANSFER-ENCODING', '').lower()
 
         if self.chunked:
-            if 'content-length' in self.headers:
-                del self.headers['content-length']
+            if 'CONTENT-LENGTH' in self.headers:
+                del self.headers['CONTENT-LENGTH']
             if 'chunked' not in te:
-                self.headers['transfer-encoding'] = 'chunked'
+                self.headers['TRANSFER-ENCODING'] = 'chunked'
 
             self.chunked = self.chunked if type(self.chunked) is int else 8196
         else:
@@ -443,13 +440,13 @@ class HttpRequest:
                 self.chunked = 8196
             else:
                 self.chunked = None
-                if 'content-length' not in self.headers:
-                    self.headers['content-length'] = str(len(self.body))
+                if 'CONTENT-LENGTH' not in self.headers:
+                    self.headers['CONTENT-LENGTH'] = str(len(self.body))
 
     def update_expect_continue(self, expect=False):
         if expect:
-            self.headers['expect'] = '100-continue'
-        elif self.headers.get('expect', '').lower() == '100-continue':
+            self.headers['EXPECT'] = '100-continue'
+        elif self.headers.get('EXPECT', '').lower() == '100-continue':
             expect = True
 
         if expect:
@@ -497,7 +494,7 @@ class HttpRequest:
                             'Bytes object is expected, got: %s.' %
                             type(result))
             else:
-                if isinstance(self.body, bytes):
+                if isinstance(self.body, (bytes, bytearray)):
                     self.body = (self.body,)
 
                 for chunk in self.body:
@@ -518,13 +515,16 @@ class HttpRequest:
         if self.chunked is not None:
             request.add_chunking_filter(self.chunked)
 
-        request.add_headers(*self.headers.items())
+        request.add_headers(
+            *((k, v)
+              for k, v in ((k, value)
+                           for k, value in self.headers.items(getall=True))))
         request.send_headers()
 
         self._writer = asyncio.async(
             self.write_bytes(request, reader), loop=self.loop)
 
-        self.response = HttpResponse(
+        self.response = self.response_class(
             self.method, self.path, self.host,
             writer=self._writer, continue100=self._continue)
         return self.response
@@ -538,7 +538,7 @@ class HttpRequest:
                 self._writer = None
 
 
-class HttpResponse(http.client.HTTPMessage):
+class ClientResponse:
 
     message = None  # RawResponseMessage object
 
@@ -562,6 +562,7 @@ class HttpResponse(http.client.HTTPMessage):
         self.method = method
         self.url = url
         self.host = host
+        self.headers = None
         self._content = None
         self._writer = writer
         if writer is not None:
@@ -570,9 +571,9 @@ class HttpResponse(http.client.HTTPMessage):
 
     def __repr__(self):
         out = io.StringIO()
-        print('<HttpResponse({}{}) [{} {}]>'.format(
+        print('<ClientResponse({}{}) [{} {}]>'.format(
             self.host, self.url, self.status, self.reason), file=out)
-        print(super().__str__(), file=out)
+        print(self.headers, file=out)
         return out.getvalue()
 
     __str__ = __repr__
@@ -584,7 +585,7 @@ class HttpResponse(http.client.HTTPMessage):
         self._reader = connection.reader
         self.connection = connection
 
-        msg = ('HttpResponse has to be closed explicitly! {}:{}:{}'
+        msg = ('ClientResponse has to be closed explicitly! {}:{}:{}'
                .format(self.method, self.host, self.url))
 
         def _do_close_connection(wr, connection=connection, msg=msg):
@@ -616,8 +617,8 @@ class HttpResponse(http.client.HTTPMessage):
         self.reason = self.message.reason
 
         # headers
-        for hdr, val in self.message.headers:
-            self.add_header(hdr, val)
+        self.headers = CaseInsensitiveMultiDict(
+            self.message.headers.items(getall=True))
 
         # payload
         self.content = self._reader.set_parser(
@@ -625,8 +626,8 @@ class HttpResponse(http.client.HTTPMessage):
 
         # cookies
         self.cookies = http.cookies.SimpleCookie()
-        if 'Set-Cookie' in self:
-            for hdr in self.get_all('Set-Cookie'):
+        if 'SET-COOKIE' in self.headers:
+            for hdr in self.headers.getall('SET-COOKIE'):
                 try:
                     self.cookies.load(hdr)
                 except http.cookies.CookieError as exc:
@@ -683,7 +684,7 @@ class HttpResponse(http.client.HTTPMessage):
         data = self._content
 
         if decode:
-            ct = self.get('content-type', '').lower()
+            ct = self.headers.get('CONTENT-TYPE', '').lower()
             if ct == 'application/json':
                 data = json.loads(data.decode('utf-8'))
 
@@ -770,7 +771,7 @@ class HttpClient:
     """
 
     def __init__(self, hosts, *, method=None, path=None,
-                 ssl=False, timeout=None,
+                 ssl=False,
                  conn_pool=True, conn_timeout=None, failed_timeout=5.0,
                  resolve=True, resolve_timeout=360.0, verify_ssl=True,
                  loop=None):
@@ -784,23 +785,25 @@ class HttpClient:
 
         self._hosts = []
         for host in hosts:
+            has_port = False
             if isinstance(host, str):
                 if ':' in host:
                     host, port = host.split(':')
                     try:
                         port = int(port)
+                        has_port = True
                     except:
                         raise ValueError('Port has to be integer: %s' % host)
                 else:
                     port = 80
             else:
+                has_port = True
                 host, port = host
 
-            self._hosts.append((host, port))
+            self._hosts.append((host, port, has_port))
 
         self._method = method
         self._path = path
-        self._timeout = timeout
         self._schema = 'https' if ssl else 'http'
 
         self._failed = collections.deque()
@@ -857,7 +860,6 @@ class HttpClient:
                 encoding='utf-8',
                 version=(1, 1),
                 compress=None,
-                timeout=None,
                 chunked=None,
                 expect100=False,
                 read_until_eof=True):
@@ -866,21 +868,24 @@ class HttpClient:
             method = self._method
         if path is None:
             path = self._path
-        if timeout is None:
-            timeout = self._timeout
 
         # if all hosts marked as failed try first from failed
         if not self._hosts:
             self._hosts.append(self._failed.popleft()[0])
 
+        url = ''
         hosts = self._hosts
 
         while hosts:
             idx = random.randint(0, len(hosts)-1)
 
             info = hosts[idx]
-            url = urllib.parse.urljoin(
-                '{}://{}:{}'.format(self._schema, info[0], info[1]), path)
+            if info[2]:
+                url = urllib.parse.urljoin(
+                    '{}://{}:{}'.format(self._schema, info[0], info[1]), path)
+            else:
+                url = urllib.parse.urljoin(
+                    '{}://{}'.format(self._schema, info[0]), path)
 
             try:
                 resp = yield from request(
@@ -888,7 +893,7 @@ class HttpClient:
                     cookies=cookies, files=files, auth=auth,
                     encoding=encoding, allow_redirects=allow_redirects,
                     version=version, max_redirects=max_redirects,
-                    timeout=timeout, compress=compress, chunked=chunked,
+                    compress=compress, chunked=chunked,
                     expect100=expect100, read_until_eof=read_until_eof,
                     connector=self._connector, loop=self._loop)
             except (aiohttp.ConnectionError, aiohttp.TimeoutError):
@@ -910,4 +915,9 @@ class HttpClient:
                 if self._connector:
                     self._connector.clear_resolved_hosts(info[0], info[1])
 
-        raise aiohttp.ConnectionError('All hosts are unreachable.')
+        raise aiohttp.ConnectionError('All hosts are unreachable %s' % url)
+
+
+# backward compatibility
+HttpRequest = ClientRequest
+HttpResponse = ClientResponse
