@@ -17,6 +17,7 @@ from wsgiref.handlers import format_date_time
 
 import aiohttp
 from aiohttp import errors
+from aiohttp import multidict
 
 METHRE = re.compile('[A-Z0-9$-_.]+')
 VERSRE = re.compile('HTTP/(\d+).(\d+)')
@@ -54,7 +55,7 @@ class HttpParser:
         """
         close_conn = None
         encoding = None
-        headers = collections.deque()
+        headers = []
 
         lines_idx = 1
         line = lines[1]
@@ -114,7 +115,7 @@ class HttpParser:
 
             headers.append((name, value))
 
-        return headers, close_conn, encoding
+        return multidict.MultiDict(headers), close_conn, encoding
 
 
 class HttpPrefixParser:
@@ -124,25 +125,20 @@ class HttpPrefixParser:
         self.allowed_methods = [m.upper() for m in allowed_methods]
 
     def __call__(self, out, buf):
-        try:
-            raw_data = yield from buf.waituntil(b' ', 24)
-            method = raw_data.decode('ascii', 'surrogateescape').strip()
+        raw_data = yield from buf.waituntil(b' ', 24)
+        method = raw_data.decode('ascii', 'surrogateescape').strip()
 
-            # method
-            method = method.upper()
-            if not METHRE.match(method):
-                raise errors.BadStatusLine(method)
+        # method
+        method = method.upper()
+        if not METHRE.match(method):
+            raise errors.BadStatusLine(method)
 
-            # allowed method
-            if self.allowed_methods and method not in self.allowed_methods:
-                raise errors.HttpMethodNotAllowed(method)
+        # allowed method
+        if self.allowed_methods and method not in self.allowed_methods:
+            raise errors.HttpMethodNotAllowed(method)
 
-            out.feed_data(method)
-            out.feed_eof()
-        except aiohttp.EofStream:
-            # Presumably, the server closed the connection before
-            # sending a valid response.
-            pass
+        out.feed_data(method)
+        out.feed_eof()
 
 
 class HttpRequestParser(HttpParser):
@@ -152,46 +148,45 @@ class HttpRequestParser(HttpParser):
     """
 
     def __call__(self, out, buf):
+        # read http message (request line + headers)
         try:
-            # read http message (request line + headers)
             raw_data = yield from buf.readuntil(
-                b'\r\n\r\n', self.max_headers, errors.LineTooLong)
-            lines = raw_data.decode(
-                'ascii', 'surrogateescape').splitlines(True)
+                b'\r\n\r\n', self.max_headers)
+        except errors.LineLimitExceededParserError as exc:
+            raise errors.LineTooLong(exc.limit) from None
 
-            # request line
-            line = lines[0]
-            try:
-                method, path, version = line.split(None, 2)
-            except ValueError:
-                raise errors.BadStatusLine(line) from None
+        lines = raw_data.decode(
+            'ascii', 'surrogateescape').splitlines(True)
 
-            # method
-            method = method.upper()
-            if not METHRE.match(method):
-                raise errors.BadStatusLine(method)
+        # request line
+        line = lines[0]
+        try:
+            method, path, version = line.split(None, 2)
+        except ValueError:
+            raise errors.BadStatusLine(line) from None
 
-            # version
-            match = VERSRE.match(version)
-            if match is None:
-                raise errors.BadStatusLine(version)
-            version = (int(match.group(1)), int(match.group(2)))
+        # method
+        method = method.upper()
+        if not METHRE.match(method):
+            raise errors.BadStatusLine(method)
 
-            # read headers
-            headers, close, compression = self.parse_headers(lines)
-            if version <= (1, 0):
-                close = True
-            elif close is None:
-                close = False
+        # version
+        match = VERSRE.match(version)
+        if match is None:
+            raise errors.BadStatusLine(version)
+        version = (int(match.group(1)), int(match.group(2)))
 
-            out.feed_data(
-                RawRequestMessage(
-                    method, path, version, headers, close, compression))
-            out.feed_eof()
-        except aiohttp.EofStream:
-            # Presumably, the server closed the connection before
-            # sending a valid response.
-            pass
+        # read headers
+        headers, close, compression = self.parse_headers(lines)
+        if version <= (1, 0):
+            close = True
+        elif close is None:
+            close = False
+
+        out.feed_data(
+            RawRequestMessage(
+                method, path, version, headers, close, compression))
+        out.feed_eof()
 
 
 class HttpResponseParser(HttpParser):
@@ -203,9 +198,12 @@ class HttpResponseParser(HttpParser):
     def __call__(self, out, buf):
         try:
             # read http message (response line + headers)
-            raw_data = yield from buf.readuntil(
-                b'\r\n\r\n', self.max_line_size+self.max_headers,
-                errors.LineTooLong)
+            try:
+                raw_data = yield from buf.readuntil(
+                    b'\r\n\r\n', self.max_line_size+self.max_headers)
+            except errors.LineLimitExceededParserError as exc:
+                raise errors.LineTooLong(exc.limit) from None
+
             lines = raw_data.decode(
                 'ascii', 'surrogateescape').splitlines(True)
 
@@ -249,7 +247,8 @@ class HttpResponseParser(HttpParser):
         except aiohttp.EofStream:
             # Presumably, the server closed the connection before
             # sending a valid response.
-            raise errors.BadStatusLine(b'') from None
+            raise errors.ClientConnectionError(
+                'Can not read status line') from None
 
 
 class HttpPayloadParser:
@@ -262,22 +261,16 @@ class HttpPayloadParser:
 
     def __call__(self, out, buf):
         # payload params
-        chunked = False
-        length = self.length
-        for name, value in self.message.headers:
-            if name == 'CONTENT-LENGTH':
-                length = value
-            elif name == 'TRANSFER-ENCODING':
-                chunked = value.lower() == 'chunked'
-            elif name == 'SEC-WEBSOCKET-KEY1':
-                length = 8
+        length = self.message.headers.get('CONTENT-LENGTH', self.length)
+        if 'SEC-WEBSOCKET-KEY1' in self.message.headers:
+            length = 8
 
         # payload decompression wrapper
         if self.compression and self.message.compression:
             out = DeflateBuffer(out, self.message.compression)
 
         # payload parser
-        if chunked:
+        if 'chunked' in self.message.headers.get('TRANSFER-ENCODING', ''):
             yield from self.parse_chunked_payload(out, buf)
 
         elif length is not None:
@@ -332,22 +325,26 @@ class HttpPayloadParser:
             yield from buf.skipuntil(b'\r\n')
 
         except aiohttp.EofStream:
-            raise errors.IncompleteRead(b'') from None
+            raise errors.ConnectionError('Broken chunked payload.') from None
 
-    def parse_length_payload(self, out, buf, length):
+    def parse_length_payload(self, out, buf, length=0):
         """Read specified amount of bytes."""
+        required = length
         try:
-            while length:
-                chunk = yield from buf.readsome(length)
+            while required:
+                chunk = yield from buf.readsome(required)
                 out.feed_data(chunk)
-                length -= len(chunk)
+                required -= len(chunk)
         except aiohttp.EofStream:
-            raise errors.IncompleteRead(b'') from None
+            raise errors.IncompleteRead(required, length-required)
 
     def parse_eof_payload(self, out, buf):
         """Read all bytes untile eof."""
-        while True:
-            out.feed_data((yield from buf.readsome()))
+        try:
+            while True:
+                out.feed_data((yield from buf.readsome()))
+        except aiohttp.EofStream:
+            pass
 
 
 class DeflateBuffer:
@@ -537,7 +534,7 @@ class HttpMessage:
 
         self.chunked = False
         self.length = None
-        self.headers = collections.deque()
+        self.headers = multidict.CaseInsensitiveMutableMultiDict()
         self.headers_sent = False
         self.output_length = 0
         self._output_size = 0
@@ -563,8 +560,10 @@ class HttpMessage:
         removes hop headers, etc."""
         assert not self.headers_sent, 'headers have been sent already'
         assert isinstance(name, str), '{!r} is not a string'.format(name)
+        assert isinstance(value, str), '{!r} is not a string'.format(value)
 
         name = name.strip().upper()
+        value = value.strip()
 
         if name == 'CONTENT-LENGTH':
             self.length = int(value)
@@ -583,7 +582,7 @@ class HttpMessage:
         elif name == 'UPGRADE':
             if 'websocket' in value.lower():
                 self.websocket = True
-                self.headers.append((name, value))
+                self.headers[name] = value
 
         elif name == 'TRANSFER-ENCODING' and not self.chunked:
             self.chunked = value.lower().strip() == 'chunked'
@@ -593,7 +592,7 @@ class HttpMessage:
                 self._has_user_agent = True
 
             # ignore hopbyhop headers
-            self.headers.append((name, value))
+            self.headers.add(name, value)
 
     def add_headers(self, *headers):
         """Adds headers to a http message."""
@@ -629,8 +628,10 @@ class HttpMessage:
         # status + headers
         hdrs = ''.join(itertools.chain(
             (self.status_line,),
-            *((k, ': ', v, '\r\n') for k, v in self.headers)))
-        hdrs = hdrs.encode('ascii') + b'\r\n'
+            *((k, ': ', v, '\r\n')
+              for k, v in ((k, value)
+                           for k, value in self.headers.items(getall=True)))))
+        hdrs = hdrs.encode('utf-8') + b'\r\n'
 
         self.output_length += len(hdrs)
         self.transport.write(hdrs)
@@ -645,9 +646,9 @@ class HttpMessage:
             connection = 'close'
 
         if self.chunked:
-            self.headers.appendleft(('TRANSFER-ENCODING', 'chunked'))
+            self.headers['TRANSFER-ENCODING'] = 'chunked'
 
-        self.headers.appendleft(('CONNECTION', connection))
+        self.headers['CONNECTION'] = connection
 
     def write(self, chunk):
         """write() writes chunk of data to a steram by using different writers.
@@ -809,6 +810,7 @@ class Response(HttpMessage):
 
     def _add_default_headers(self):
         super()._add_default_headers()
+
         self.headers.extend((('DATE', format_date_time(None)),
                              ('SERVER', self.SERVER_SOFTWARE),))
 
@@ -828,5 +830,6 @@ class Request(HttpMessage):
 
     def _add_default_headers(self):
         super()._add_default_headers()
+
         if not self._has_user_agent:
-            self.headers.append(('USER-AGENT', self.SERVER_SOFTWARE))
+            self.headers['USER-AGENT'] = self.SERVER_SOFTWARE
