@@ -41,10 +41,8 @@ class HttpRequestHandler(aiohttp.server.ServerHttpProtocol):
         if not path:
             raise aiohttp.HttpErrorException(404)
 
-        headers = email.message.Message()
-        for hdr, val in message.headers:
+        for hdr, val in message.headers.items(getall=True):
             print(hdr, val)
-            headers.add_header(hdr, val)
 
         if isdir and not path.endswith('/'):
             path = path + '/'
@@ -56,7 +54,7 @@ class HttpRequestHandler(aiohttp.server.ServerHttpProtocol):
         response.add_header('Transfer-Encoding', 'chunked')
 
         # content encoding
-        accept_encoding = headers.get('accept-encoding', '').lower()
+        accept_encoding = message.headers.get('accept-encoding', '').lower()
         if 'deflate' in accept_encoding:
             response.add_header('Content-Encoding', 'deflate')
             response.add_compression_filter('deflate')
