@@ -9,10 +9,31 @@ import unittest.mock
 import urllib.parse
 
 import aiohttp
-from aiohttp.client import HttpRequest, HttpResponse, HttpClient
+from aiohttp.client import ClientRequest, ClientResponse, HttpClient
 
 
-class HttpResponseTests(unittest.TestCase):
+class RequestTests(unittest.TestCase):
+
+    def setUp(self):
+        self.loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(None)
+
+    def tearDown(self):
+        self.loop.close()
+
+    def test_bad_status_response(self):
+        connector = unittest.mock.Mock()
+        connector.connect.side_effect = aiohttp.BadStatusLine
+
+        self.assertRaises(
+            aiohttp.ClientConnectionError,
+            self.loop.run_until_complete,
+            aiohttp.request(
+                'get', 'http://example.com',
+                connector=connector, loop=self.loop))
+
+
+class ClientResponseTests(unittest.TestCase):
 
     def setUp(self):
         self.loop = asyncio.new_event_loop()
@@ -20,13 +41,13 @@ class HttpResponseTests(unittest.TestCase):
 
         self.connection = unittest.mock.Mock()
         self.stream = aiohttp.StreamParser(loop=self.loop)
-        self.response = HttpResponse('get', 'http://python.org')
+        self.response = ClientResponse('get', 'http://python.org')
 
     def tearDown(self):
         self.loop.close()
 
     def test_del(self):
-        response = HttpResponse('get', 'http://python.org')
+        response = ClientResponse('get', 'http://python.org')
 
         connection = unittest.mock.Mock()
         response._setup_connection(connection)
@@ -43,10 +64,10 @@ class HttpResponseTests(unittest.TestCase):
         self.response.close()
 
     def test_wait_for_100(self):
-        response = HttpResponse(
+        response = ClientResponse(
             'get', 'http://python.org', continue100=object())
         self.assertTrue(response.waiting_for_continue())
-        response = HttpResponse(
+        response = ClientResponse(
             'get', 'http://python.org')
         self.assertFalse(response.waiting_for_continue())
 
@@ -54,7 +75,8 @@ class HttpResponseTests(unittest.TestCase):
         self.response.status = 200
         self.response.reason = 'Ok'
         self.assertIn(
-            '<HttpResponse(http://python.org) [200 Ok]>', repr(self.response))
+            '<ClientResponse(http://python.org) [200 Ok]>',
+            repr(self.response))
 
     def test_read_and_close(self):
         self.response.read = unittest.mock.Mock()
@@ -80,7 +102,7 @@ class HttpResponseTests(unittest.TestCase):
         self.response.close.assert_called_with(True)
 
 
-class HttpRequestTests(unittest.TestCase):
+class ClientRequestTests(unittest.TestCase):
 
     def setUp(self):
         self.loop = asyncio.new_event_loop()
@@ -96,224 +118,232 @@ class HttpRequestTests(unittest.TestCase):
         self.loop.close()
 
     def test_method(self):
-        req = HttpRequest('get', 'http://python.org/')
+        req = ClientRequest('get', 'http://python.org/')
         self.assertEqual(req.method, 'GET')
 
-        req = HttpRequest('head', 'http://python.org/')
+        req = ClientRequest('head', 'http://python.org/')
         self.assertEqual(req.method, 'HEAD')
 
-        req = HttpRequest('HEAD', 'http://python.org/')
+        req = ClientRequest('HEAD', 'http://python.org/')
         self.assertEqual(req.method, 'HEAD')
 
     def test_version(self):
-        req = HttpRequest('get', 'http://python.org/', version='1.0')
+        req = ClientRequest('get', 'http://python.org/', version='1.0')
         self.assertEqual(req.version, (1, 0))
 
     def test_version_err(self):
         self.assertRaises(
             ValueError,
-            HttpRequest, 'get', 'http://python.org/', version='1.c')
+            ClientRequest, 'get', 'http://python.org/', version='1.c')
 
     def test_host_port(self):
-        req = HttpRequest('get', 'http://python.org/')
+        req = ClientRequest('get', 'http://python.org/')
         self.assertEqual(req.host, 'python.org')
         self.assertEqual(req.port, 80)
         self.assertFalse(req.ssl)
 
-        req = HttpRequest('get', 'https://python.org/')
+        req = ClientRequest('get', 'https://python.org/')
         self.assertEqual(req.host, 'python.org')
         self.assertEqual(req.port, 443)
         self.assertTrue(req.ssl)
 
-        req = HttpRequest('get', 'https://python.org:960/')
+        req = ClientRequest('get', 'https://python.org:960/')
         self.assertEqual(req.host, 'python.org')
         self.assertEqual(req.port, 960)
         self.assertTrue(req.ssl)
 
     def test_host_port_err(self):
         self.assertRaises(
-            ValueError, HttpRequest, 'get', 'http://python.org:123e/')
+            ValueError, ClientRequest, 'get', 'http://python.org:123e/')
 
     def test_host_header(self):
-        req = HttpRequest('get', 'http://python.org/')
-        self.assertEqual(req.headers['host'], 'python.org')
+        req = ClientRequest('get', 'http://python.org/')
+        self.assertEqual(req.headers['HOST'], 'python.org')
 
-        req = HttpRequest('get', 'http://python.org/',
-                          headers={'host': 'example.com'})
-        self.assertEqual(req.headers['host'], 'example.com')
+        req = ClientRequest('get', 'http://python.org:80/')
+        self.assertEqual(req.headers['HOST'], 'python.org:80')
+
+        req = ClientRequest('get', 'http://python.org:99/')
+        self.assertEqual(req.headers['HOST'], 'python.org:99')
+
+        req = ClientRequest('get', 'http://python.org/',
+                            headers={'host': 'example.com'})
+        self.assertEqual(req.headers['HOST'], 'example.com')
+
+        req = ClientRequest('get', 'http://python.org/',
+                            headers={'host': 'example.com:99'})
+        self.assertEqual(req.headers['HOST'], 'example.com:99')
 
     def test_headers(self):
-        req = HttpRequest('get', 'http://python.org/',
-                          headers={'Content-Type': 'text/plain'})
-        self.assertIn('Content-Type', req.headers)
-        self.assertEqual(req.headers['Content-Type'], 'text/plain')
-        self.assertEqual(req.headers['Accept-Encoding'], 'gzip, deflate')
+        req = ClientRequest('get', 'http://python.org/',
+                            headers={'Content-Type': 'text/plain'})
+        self.assertIn('CONTENT-TYPE', req.headers)
+        self.assertEqual(req.headers['CONTENT-TYPE'], 'text/plain')
+        self.assertEqual(req.headers['ACCEPT-ENCODING'], 'gzip, deflate')
 
     def test_headers_list(self):
-        req = HttpRequest('get', 'http://python.org/',
-                          headers=[('Content-Type', 'text/plain')])
-        self.assertIn('Content-Type', req.headers)
-        self.assertEqual(req.headers['Content-Type'], 'text/plain')
+        req = ClientRequest('get', 'http://python.org/',
+                            headers=[('Content-Type', 'text/plain')])
+        self.assertIn('CONTENT-TYPE', req.headers)
+        self.assertEqual(req.headers['CONTENT-TYPE'], 'text/plain')
 
     def test_headers_default(self):
-        req = HttpRequest('get', 'http://python.org/',
-                          headers={'Accept-Encoding': 'deflate'})
-        self.assertEqual(req.headers['Accept-Encoding'], 'deflate')
+        req = ClientRequest('get', 'http://python.org/',
+                            headers={'ACCEPT-ENCODING': 'deflate'})
+        self.assertEqual(req.headers['ACCEPT-ENCODING'], 'deflate')
 
     def test_invalid_url(self):
-        self.assertRaises(ValueError, HttpRequest, 'get', 'hiwpefhipowhefopw')
+        self.assertRaises(
+            ValueError, ClientRequest, 'get', 'hiwpefhipowhefopw')
 
     def test_invalid_idna(self):
         self.assertRaises(
-            ValueError, HttpRequest, 'get', 'http://\u2061owhefopw.com')
+            ValueError, ClientRequest, 'get', 'http://\u2061owhefopw.com')
 
     def test_no_path(self):
-        req = HttpRequest('get', 'http://python.org')
+        req = ClientRequest('get', 'http://python.org')
         self.assertEqual('/', req.path)
 
-    def test_path_with_proxy(self):
-        req = HttpRequest('get', 'http://python.org', using_proxy=True)
-        self.assertEqual('http://python.org/', req.path)
-
     def test_basic_auth(self):
-        req = HttpRequest('get', 'http://python.org', auth=('nkim', '1234'))
-        self.assertIn('Authorization', req.headers)
-        self.assertEqual('Basic bmtpbToxMjM0', req.headers['Authorization'])
+        req = ClientRequest('get', 'http://python.org', auth=('nkim', '1234'))
+        self.assertIn('AUTHORIZATION', req.headers)
+        self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
     def test_basic_auth_from_url(self):
-        req = HttpRequest('get', 'http://nkim:1234@python.org')
-        self.assertIn('Authorization', req.headers)
-        self.assertEqual('Basic bmtpbToxMjM0', req.headers['Authorization'])
+        req = ClientRequest('get', 'http://nkim:1234@python.org')
+        self.assertIn('AUTHORIZATION', req.headers)
+        self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
-        req = HttpRequest('get', 'http://nkim@python.org')
-        self.assertIn('Authorization', req.headers)
-        self.assertEqual('Basic bmtpbTo=', req.headers['Authorization'])
+        req = ClientRequest('get', 'http://nkim@python.org')
+        self.assertIn('AUTHORIZATION', req.headers)
+        self.assertEqual('Basic bmtpbTo=', req.headers['AUTHORIZATION'])
 
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://nkim@python.org', auth=('nkim', '1234'))
-        self.assertIn('Authorization', req.headers)
-        self.assertEqual('Basic bmtpbToxMjM0', req.headers['Authorization'])
+        self.assertIn('AUTHORIZATION', req.headers)
+        self.assertEqual('Basic bmtpbToxMjM0', req.headers['AUTHORIZATION'])
 
     def test_basic_auth_err(self):
         self.assertRaises(
-            ValueError, HttpRequest,
+            ValueError, ClientRequest,
             'get', 'http://python.org', auth=(1, 2, 3))
 
     def test_no_content_length(self):
-        req = HttpRequest('get', 'http://python.org', loop=self.loop)
+        req = ClientRequest('get', 'http://python.org', loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('0', req.headers.get('Content-Length'))
+        self.assertEqual('0', req.headers.get('CONTENT-LENGTH'))
 
-        req = HttpRequest('head', 'http://python.org', loop=self.loop)
+        req = ClientRequest('head', 'http://python.org', loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('0', req.headers.get('Content-Length'))
+        self.assertEqual('0', req.headers.get('CONTENT-LENGTH'))
 
     def test_path_is_not_double_encoded(self):
-        req = HttpRequest('get', "http://0.0.0.0/get/test case")
+        req = ClientRequest('get', "http://0.0.0.0/get/test case")
         self.assertEqual(req.path, "/get/test%20case")
 
-        req = HttpRequest('get', "http://0.0.0.0/get/test%2fcase")
+        req = ClientRequest('get', "http://0.0.0.0/get/test%2fcase")
         self.assertEqual(req.path, "/get/test%2fcase")
 
-        req = HttpRequest('get', "http://0.0.0.0/get/test%20case")
+        req = ClientRequest('get', "http://0.0.0.0/get/test%20case")
         self.assertEqual(req.path, "/get/test%20case")
 
     def test_params_are_added_before_fragment(self):
-        req = HttpRequest(
+        req = ClientRequest(
             'GET', "http://example.com/path#fragment", params={"a": "b"})
         self.assertEqual(
             req.path, "/path?a=b#fragment")
 
-        req = HttpRequest(
+        req = ClientRequest(
             'GET',
             "http://example.com/path?key=value#fragment", params={"a": "b"})
         self.assertEqual(
             req.path, "/path?key=value&a=b#fragment")
 
     def test_cookies(self):
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://test.com/path', cookies={'cookie1': 'val1'})
-        self.assertIn('Cookie', req.headers)
-        self.assertEqual('cookie1=val1', req.headers['cookie'])
+        self.assertIn('COOKIE', req.headers)
+        self.assertEqual('cookie1=val1', req.headers['COOKIE'])
 
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://test.com/path',
             headers={'cookie': 'cookie1=val1'},
             cookies={'cookie2': 'val2'})
-        self.assertEqual('cookie1=val1; cookie2=val2', req.headers['cookie'])
+        self.assertEqual('cookie1=val1; cookie2=val2', req.headers['COOKIE'])
 
     def test_unicode_get(self):
         def join(*suffix):
             return urllib.parse.urljoin('http://python.org/', '/'.join(suffix))
 
         url = 'http://python.org'
-        req = HttpRequest('get', url, params={'foo': 'f\xf8\xf8'})
+        req = ClientRequest('get', url, params={'foo': 'f\xf8\xf8'})
         self.assertEqual('/?foo=f%C3%B8%C3%B8', req.path)
-        req = HttpRequest('', url, params={'f\xf8\xf8': 'f\xf8\xf8'})
+        req = ClientRequest('', url, params={'f\xf8\xf8': 'f\xf8\xf8'})
         self.assertEqual('/?f%C3%B8%C3%B8=f%C3%B8%C3%B8', req.path)
-        req = HttpRequest('', url, params={'foo': 'foo'})
+        req = ClientRequest('', url, params={'foo': 'foo'})
         self.assertEqual('/?foo=foo', req.path)
-        req = HttpRequest('', join('\xf8'), params={'foo': 'foo'})
+        req = ClientRequest('', join('\xf8'), params={'foo': 'foo'})
         self.assertEqual('/%C3%B8?foo=foo', req.path)
 
     def test_query_multivalued_param(self):
-        for meth in HttpRequest.ALL_METHODS:
-            req = HttpRequest(
+        for meth in ClientRequest.ALL_METHODS:
+            req = ClientRequest(
                 meth, 'http://python.org',
                 params=(('test', 'foo'), ('test', 'baz')))
             self.assertEqual(req.path, '/?test=foo&test=baz')
 
     def test_post_data(self):
-        for meth in HttpRequest.POST_METHODS:
-            req = HttpRequest(
+        for meth in ClientRequest.POST_METHODS:
+            req = ClientRequest(
                 meth, 'http://python.org/',
                 data={'life': '42'}, loop=self.loop)
             req.send(self.transport, self.protocol)
             self.assertEqual('/', req.path)
             self.assertEqual(b'life=42', req.body)
             self.assertEqual('application/x-www-form-urlencoded',
-                             req.headers['content-type'])
+                             req.headers['CONTENT-TYPE'])
 
     def test_get_with_data(self):
-        for meth in HttpRequest.GET_METHODS:
-            req = HttpRequest(meth, 'http://python.org/', data={'life': '42'})
+        for meth in ClientRequest.GET_METHODS:
+            req = ClientRequest(
+                meth, 'http://python.org/', data={'life': '42'})
             self.assertEqual('/?life=42', req.path)
 
     def test_bytes_data(self):
-        for meth in HttpRequest.POST_METHODS:
-            req = HttpRequest(
+        for meth in ClientRequest.POST_METHODS:
+            req = ClientRequest(
                 meth, 'http://python.org/',
                 data=b'binary data', loop=self.loop)
             req.send(self.transport, self.protocol)
             self.assertEqual('/', req.path)
             self.assertEqual(b'binary data', req.body)
             self.assertEqual('application/octet-stream',
-                             req.headers['content-type'])
+                             req.headers['CONTENT-TYPE'])
 
     def test_files_and_bytes_data(self):
         self.assertRaises(
-            NotImplementedError, HttpRequest,
+            NotImplementedError, ClientRequest,
             'POST', 'http://python.org/',
             data=b'binary data', files={'file': b'file data'})
 
     @unittest.mock.patch('aiohttp.client.aiohttp')
     def test_content_encoding(self, m_http):
-        req = HttpRequest('get', 'http://python.org/',
-                          compress='deflate', loop=self.loop)
+        req = ClientRequest('get', 'http://python.org/',
+                            compress='deflate', loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual(req.headers['Transfer-encoding'], 'chunked')
-        self.assertEqual(req.headers['Content-encoding'], 'deflate')
+        self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
+        self.assertEqual(req.headers['CONTENT-ENCODING'], 'deflate')
         m_http.Request.return_value\
             .add_compression_filter.assert_called_with('deflate')
 
     @unittest.mock.patch('aiohttp.client.aiohttp')
     def test_content_encoding_header(self, m_http):
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://python.org/',
             headers={'Content-Encoding': 'deflate'}, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual(req.headers['Transfer-encoding'], 'chunked')
-        self.assertEqual(req.headers['Content-encoding'], 'deflate')
+        self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
+        self.assertEqual(req.headers['CONTENT-ENCODING'], 'deflate')
 
         m_http.Request.return_value\
             .add_compression_filter.assert_called_with('deflate')
@@ -321,56 +351,56 @@ class HttpRequestTests(unittest.TestCase):
             .add_chunking_filter.assert_called_with(8196)
 
     def test_chunked(self):
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://python.org/',
-            headers={'Transfer-encoding': 'gzip'}, loop=self.loop)
+            headers={'TRANSFER-ENCODING': 'gzip'}, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('gzip', req.headers['Transfer-encoding'])
+        self.assertEqual('gzip', req.headers['TRANSFER-ENCODING'])
 
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://python.org/',
             headers={'Transfer-encoding': 'chunked'}, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('chunked', req.headers['Transfer-encoding'])
+        self.assertEqual('chunked', req.headers['TRANSFER-ENCODING'])
 
     @unittest.mock.patch('aiohttp.client.aiohttp')
     def test_chunked_explicit(self, m_http):
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://python.org/', chunked=True, loop=self.loop)
         req.send(self.transport, self.protocol)
 
-        self.assertEqual('chunked', req.headers['Transfer-encoding'])
+        self.assertEqual('chunked', req.headers['TRANSFER-ENCODING'])
         m_http.Request.return_value\
                       .add_chunking_filter.assert_called_with(8196)
 
     @unittest.mock.patch('aiohttp.client.aiohttp')
     def test_chunked_explicit_size(self, m_http):
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://python.org/', chunked=1024, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('chunked', req.headers['Transfer-encoding'])
+        self.assertEqual('chunked', req.headers['TRANSFER-ENCODING'])
         m_http.Request.return_value\
                       .add_chunking_filter.assert_called_with(1024)
 
     def test_chunked_length(self):
-        req = HttpRequest(
+        req = ClientRequest(
             'get', 'http://python.org/',
-            headers={'Content-Length': '1000'}, chunked=1024, loop=self.loop)
+            headers={'CONTENT-LENGTH': '1000'}, chunked=1024, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual(req.headers['Transfer-Encoding'], 'chunked')
-        self.assertNotIn('Content-Length', req.headers)
+        self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
+        self.assertNotIn('CONTENT-LENGTH', req.headers)
 
     def test_expect100(self):
-        req = HttpRequest('get', 'http://python.org/',
-                          expect100=True, loop=self.loop)
+        req = ClientRequest('get', 'http://python.org/',
+                            expect100=True, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('100-continue', req.headers['expect'])
+        self.assertEqual('100-continue', req.headers['EXPECT'])
         self.assertIsNotNone(req._continue)
 
-        req = HttpRequest('get', 'http://python.org/',
-                          headers={'expect': '100-continue'}, loop=self.loop)
+        req = ClientRequest('get', 'http://python.org/',
+                            headers={'expect': '100-continue'}, loop=self.loop)
         req.send(self.transport, self.protocol)
-        self.assertEqual('100-continue', req.headers['expect'])
+        self.assertEqual('100-continue', req.headers['EXPECT'])
         self.assertIsNotNone(req._continue)
 
     def test_data_stream(self):
@@ -378,11 +408,11 @@ class HttpRequestTests(unittest.TestCase):
             yield b'binary data'
             return b' result'
 
-        req = HttpRequest(
+        req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
         self.assertTrue(req.chunked)
         self.assertTrue(inspect.isgenerator(req.body))
-        self.assertEqual(req.headers['transfer-encoding'], 'chunked')
+        self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
 
         resp = req.send(self.transport, self.protocol)
         self.assertIsInstance(req._writer, asyncio.Future)
@@ -402,11 +432,11 @@ class HttpRequestTests(unittest.TestCase):
             yield from fut
             return b' result'
 
-        req = HttpRequest(
+        req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
         self.assertTrue(req.chunked)
         self.assertTrue(inspect.isgenerator(req.body))
-        self.assertEqual(req.headers['transfer-encoding'], 'chunked')
+        self.assertEqual(req.headers['TRANSFER-ENCODING'], 'chunked')
 
         @asyncio.coroutine
         def exc():
@@ -427,7 +457,7 @@ class HttpRequestTests(unittest.TestCase):
             yield object()
             return b' result'
 
-        req = HttpRequest(
+        req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
         req.send(self.transport, self.protocol)
         self.loop.run_until_complete(req._writer)
@@ -438,7 +468,7 @@ class HttpRequestTests(unittest.TestCase):
             yield b'binary data'
             return b' result'
 
-        req = HttpRequest(
+        req = ClientRequest(
             'POST', 'http://python.org/', data=gen(),
             expect100=True, loop=self.loop)
         self.assertTrue(req.chunked)
@@ -459,7 +489,7 @@ class HttpRequestTests(unittest.TestCase):
              unittest.mock.call(b'0\r\n\r\n')])
 
     def test_data_continue(self):
-        req = HttpRequest(
+        req = ClientRequest(
             'POST', 'http://python.org/', data=b'data',
             expect100=True, loop=self.loop)
 
@@ -483,7 +513,7 @@ class HttpRequestTests(unittest.TestCase):
             yield from asyncio.sleep(0.00001, loop=self.loop)
             return b'result'
 
-        req = HttpRequest(
+        req = ClientRequest(
             'POST', 'http://python.org/', data=gen(), loop=self.loop)
         req.send(self.transport, self.protocol)
         self.loop.run_until_complete(req.close())
@@ -493,6 +523,17 @@ class HttpRequestTests(unittest.TestCase):
              unittest.mock.call(b'\r\n'),
              unittest.mock.call(b'0\r\n\r\n')])
 
+    def test_custom_response_class(self):
+        class CustomResponse(ClientResponse):
+            def read(self, decode=False):
+                return 'customized!'
+
+        req = ClientRequest(
+            'GET', 'http://python.org/', response_class=CustomResponse,
+            loop=self.loop)
+        resp = req.send(self.transport, self.protocol)
+        self.assertEqual('customized!', resp.read())
+
 
 class HttpClientTests(unittest.TestCase):
 
@@ -509,13 +550,13 @@ class HttpClientTests(unittest.TestCase):
         self.assertRaises(ValueError, HttpClient, ('test:test',))
 
         c = HttpClient('localhost:8080', loop=self.loop)
-        self.assertEqual(c._hosts, [('localhost', 8080)])
+        self.assertEqual(c._hosts, [('localhost', 8080, True)])
 
         c = HttpClient('localhost', loop=self.loop)
-        self.assertEqual(c._hosts, [('localhost', 80)])
+        self.assertEqual(c._hosts, [('localhost', 80, False)])
 
         c = HttpClient([('localhost', 1000)], loop=self.loop)
-        self.assertEqual(c._hosts, [('localhost', 1000)])
+        self.assertEqual(c._hosts, [('localhost', 1000, True)])
 
         c = HttpClient([('localhost', 1000)])
         self.assertIs(c._loop, asyncio.get_event_loop.return_value)
@@ -579,7 +620,7 @@ class HttpClientTests(unittest.TestCase):
         self.assertRaises(
             aiohttp.ConnectionError,
             self.loop.run_until_complete,
-            c.request('get', path='/', timeout=0.0001))
+            c.request('get', path='/'))
 
     def test_failed_request_conn(self):
         c = HttpClient(
@@ -596,13 +637,13 @@ class HttpClientTests(unittest.TestCase):
         c = HttpClient(
             [('localhost', 56777), ('localhost', 56778)], loop=self.loop)
         c._hosts = []
-        c._failed.append((('localhost', 1000), now - 10))
-        c._failed.append((('localhost', 1001), now - 10))
+        c._failed.append((('localhost', 1000, False), now - 10))
+        c._failed.append((('localhost', 1001, True), now - 10))
 
         self.assertRaises(
             aiohttp.ConnectionError,
             self.loop.run_until_complete,
-            c.request('get', path='/', timeout=0.0001))
+            c.request('get', path='/'))
 
     @unittest.mock.patch('aiohttp.client.request')
     def test_cleanup_resolved_hosts_on_500(self, m_request):
