@@ -11,6 +11,7 @@ import unittest
 import aiohttp
 from aiohttp import client
 from aiohttp import test_utils
+from aiohttp.multidict import MultiDict
 
 
 class HttpClientFunctionalTests(unittest.TestCase):
@@ -513,20 +514,11 @@ class HttpClientFunctionalTests(unittest.TestCase):
             r = self.loop.run_until_complete(
                 client.request('get', httpd.url('chunked'), loop=self.loop))
             self.assertEqual(r.status, 200)
-            self.assertEqual(r['Transfer-Encoding'], 'chunked')
+            self.assertEqual(r.headers.getone('TRANSFER-ENCODING'), 'chunked')
             content = self.loop.run_until_complete(r.read(True))
             self.assertEqual(content['path'], '/chunked')
             r.close()
 
-    def test_timeout(self):
-        with test_utils.run_server(self.loop, router=Functional) as httpd:
-            httpd['noresponse'] = True
-            self.assertRaises(
-                asyncio.TimeoutError,
-                self.loop.run_until_complete,
-                client.request('get', httpd.url('method', 'get'),
-                               timeout=0.1, loop=self.loop))
-
     def test_broken_connection(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
             r = self.loop.run_until_complete(
@@ -542,7 +534,7 @@ class HttpClientFunctionalTests(unittest.TestCase):
             aiohttp.ConnectionError,
             self.loop.run_until_complete,
             client.request('get', 'http://0.0.0.0:1',
-                           timeout=0.1, loop=self.loop))
+                           loop=self.loop))
 
     def test_request_conn_closed(self):
         with test_utils.run_server(self.loop, router=Functional) as httpd:
@@ -550,8 +542,8 @@ class HttpClientFunctionalTests(unittest.TestCase):
             self.assertRaises(
                 aiohttp.ClientConnectionError,
                 self.loop.run_until_complete,
-                client.request('get', httpd.url('method', 'get'),
-                               loop=self.loop))
+                client.request(
+                    'get', httpd.url('method', 'get'), loop=self.loop))
 
     def test_keepalive(self):
         from aiohttp import connector
@@ -614,6 +606,23 @@ class HttpClientFunctionalTests(unittest.TestCase):
             self.assertEqual(
                 cookies, [('c1', 'cookie1'), ('c2', 'cookie2'), ('test', '1')])
 
+    def test_multidict_headers(self):
+        with test_utils.run_server(self.loop, router=Functional) as httpd:
+            url = httpd.url('method', 'post')
+
+            data = b'sample data'
+
+            r = self.loop.run_until_complete(
+                client.request(
+                    'post', url, data=data,
+                    headers=MultiDict({'Content-Length': str(len(data))}),
+                    loop=self.loop))
+            content = self.loop.run_until_complete(r.read(True))
+            r.close()
+
+            self.assertEqual(str(len(data)),
+                             content['headers']['Content-Length'])
+
 
 class Functional(test_utils.Router):
 
