@@ -1,6 +1,5 @@
 """Tests for aiohttp/protocol.py"""
 
-from collections import deque
 import asyncio
 import zlib
 import unittest
@@ -9,6 +8,7 @@ import unittest.mock
 import aiohttp
 from aiohttp import errors
 from aiohttp import protocol
+from aiohttp import multidict
 
 
 class ParseHeadersTests(unittest.TestCase):
@@ -24,7 +24,7 @@ class ParseHeadersTests(unittest.TestCase):
 
         headers, close, compression = self.parser.parse_headers(hdrs)
 
-        self.assertEqual(list(headers),
+        self.assertEqual(list(headers.items()),
                          [('TEST', 'line\r\n continue'), ('TEST2', 'data')])
         self.assertIsNone(close)
         self.assertIsNone(compression)
@@ -36,7 +36,7 @@ class ParseHeadersTests(unittest.TestCase):
 
         headers, close, compression = self.parser.parse_headers(hdrs)
 
-        self.assertEqual(list(headers),
+        self.assertEqual(list(headers.items(getall=True)),
                          [('SET-COOKIE', 'c1=cookie1'),
                           ('SET-COOKIE', 'c2=cookie2')])
         self.assertIsNone(close)
@@ -158,7 +158,7 @@ class ParsePayloadTests(unittest.TestCase):
         p.send(b'data')
         try:
             p.throw(aiohttp.EofStream())
-        except aiohttp.EofStream:
+        except StopIteration:
             pass
 
         self.assertEqual([b'data'], list(out._buffer))
@@ -219,7 +219,7 @@ class ParsePayloadTests(unittest.TestCase):
         p = protocol.HttpPayloadParser(None).parse_chunked_payload(out, buf)
         next(p)
         p.send(b'4\r\ndata\r\n')
-        self.assertRaises(errors.IncompleteRead, p.throw, aiohttp.EofStream)
+        self.assertRaises(errors.ConnectionError, p.throw, aiohttp.EofStream)
 
     def test_parse_chunked_payload_extension(self):
         out = aiohttp.DataQueue(self.stream)
@@ -241,7 +241,8 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_length_broken(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('CONTENT-LENGTH', 'qwe')], None, None)
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('CONTENT-LENGTH', 'qwe')]), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg)(out, buf)
@@ -249,7 +250,8 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_length_wrong(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('CONTENT-LENGTH', '-1')], None, None)
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('CONTENT-LENGTH', '-1')]), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg)(out, buf)
@@ -257,7 +259,8 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_length(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('CONTENT-LENGTH', '2')], None, None)
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('CONTENT-LENGTH', '2')]), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg)(out, buf)
@@ -272,7 +275,7 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_no_length(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [], None, None)
+            'GET', '/', (1, 1), multidict.MultiDict(), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg, readall=False)(out, buf)
@@ -285,7 +288,8 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_deflate(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('CONTENT-LENGTH', len(self._COMPRESSED))],
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('CONTENT-LENGTH', len(self._COMPRESSED))]),
             None, 'deflate')
 
         out = aiohttp.DataQueue(self.stream)
@@ -297,7 +301,8 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_deflate_disabled(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('CONTENT-LENGTH', len(self._COMPRESSED))],
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('CONTENT-LENGTH', len(self._COMPRESSED))]),
             None, 'deflate')
 
         out = aiohttp.DataQueue(self.stream)
@@ -309,7 +314,8 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_websocket(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('SEC-WEBSOCKET-KEY1', '13')], None, None)
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('SEC-WEBSOCKET-KEY1', '13')]), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg)(out, buf)
@@ -319,7 +325,9 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_chunked(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('TRANSFER-ENCODING', 'chunked')], None, None)
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('TRANSFER-ENCODING', 'chunked')]),
+            None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg)(out, buf)
@@ -330,19 +338,20 @@ class ParsePayloadTests(unittest.TestCase):
 
     def test_http_payload_parser_eof(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [], None, None)
+            'GET', '/', (1, 1), multidict.MultiDict(), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg, readall=True)(out, buf)
         next(p)
         p.send(b'data')
         p.send(b'line')
-        self.assertRaises(aiohttp.EofStream, p.throw, aiohttp.EofStream())
+        self.assertRaises(StopIteration, p.throw, aiohttp.EofStream())
         self.assertEqual(b'dataline', b''.join(out._buffer))
 
     def test_http_payload_parser_length_zero(self):
         msg = protocol.RawRequestMessage(
-            'GET', '/', (1, 1), [('CONTENT-LENGTH', '0')], None, None)
+            'GET', '/', (1, 1),
+            multidict.MultiDict([('CONTENT-LENGTH', '0')]), None, None)
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpPayloadParser(msg)(out, buf)
@@ -378,10 +387,11 @@ class ParseRequestTests(unittest.TestCase):
             pass
         result = out._buffer[0]
         self.assertEqual(
-            ('GET', '/path', (1, 1), deque(), False, None), result)
+            ('GET', '/path', (1, 1), multidict.MultiDict(), False, None),
+            result)
 
     def test_http_request_parser_eof(self):
-        # HttpRequestParser does not fail on EofStream()
+        # HttpRequestParser does fail on EofStream()
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpRequestParser()(out, buf)
@@ -389,7 +399,7 @@ class ParseRequestTests(unittest.TestCase):
         p.send(b'get /path HTTP/1.1\r\n')
         try:
             p.throw(aiohttp.EofStream())
-        except StopIteration:
+        except aiohttp.EofStream:
             pass
         self.assertFalse(out._buffer)
 
@@ -403,7 +413,8 @@ class ParseRequestTests(unittest.TestCase):
         except StopIteration:
             pass
         self.assertEqual(
-            ('GET', '//path', (1, 1), deque(), False, None), out._buffer[0])
+            ('GET', '//path', (1, 1), multidict.MultiDict(), False, None),
+            out._buffer[0])
 
     def test_http_request_parser_bad_status_line(self):
         out = aiohttp.DataQueue(self.stream)
@@ -445,13 +456,22 @@ class ParseResponseTests(unittest.TestCase):
         next(p)
         self.assertRaises(errors.BadStatusLine, p.send, b'\r\n\r\n')
 
+    def test_http_response_parser_bad_status_line_too_long(self):
+        out = aiohttp.DataQueue(self.stream)
+        buf = aiohttp.ParserBuffer()
+        p = protocol.HttpResponseParser(
+            max_headers=2, max_line_size=2)(out, buf)
+        next(p)
+        self.assertRaises(
+            errors.LineTooLong, p.send, b'HTTP/1.1 200 Ok\r\n\r\n')
+
     def test_http_response_parser_bad_status_line_eof(self):
         out = aiohttp.DataQueue(self.stream)
         buf = aiohttp.ParserBuffer()
         p = protocol.HttpResponseParser()(out, buf)
         next(p)
         self.assertRaises(
-            errors.BadStatusLine, p.throw, aiohttp.EofStream())
+            errors.ConnectionError, p.throw, aiohttp.EofStream())
 
     def test_http_response_parser_bad_version(self):
         out = aiohttp.DataQueue(self.stream)
